/**
 * The base class for exceptions (errors).
 */
export declare class Err extends Error {
    inner: Error | undefined;
    constructor(message: string, inner?: Error);
    /**
     * Returns a pretty, friendly summary for the error or for the chain of errros (if appropriate).
     */
    summary(): any[];
}
/**
 * Signals invalid arguments for a function, for an operation.
 */
export declare class ErrInvalidArgument extends Err {
    constructor(message: string, inner?: Error);
}
/**
 * Signals an unsupported operation.
 */
export declare class ErrUnsupportedOperation extends Err {
    constructor(operation: string, reason?: string);
}
/**
 * Signals the provisioning of objects of unexpected (bad) types.
 */
export declare class ErrBadType extends Err {
    constructor(name: string, type: any, value?: any);
}
/**
 * Signals that an invariant failed.
 */
export declare class ErrInvariantFailed extends Err {
    constructor(message: string);
}
/**
 * Signals an unexpected condition.
 */
export declare class ErrUnexpectedCondition extends Err {
    constructor(message: string);
}
/**
 * Signals issues with {@link Address} instantiation.
 */
export declare class ErrAddressCannotCreate extends Err {
    constructor(input: any, inner?: Error);
}
/**
 * Signals issues with the HRP of an {@link Address}.
 */
export declare class ErrAddressBadHrp extends Err {
    constructor(expected: string, got: string);
}
/**
 * Signals the presence of an empty / invalid address.
 */
export declare class ErrAddressEmpty extends Err {
    constructor();
}
/**
 * Signals an invalid value for gas limit.
 */
export declare class ErrNotEnoughGas extends Err {
    constructor(value: number);
}
/**
 * Signals an invalid value for nonce.
 */
export declare class ErrNonceInvalid extends Err {
    constructor(value: number);
}
/**
 * Signals an invalid value for transaction version.
 */
export declare class ErrTransactionVersionInvalid extends Err {
    constructor(value: number);
}
/**
 * Signals an invalid value for transaction options.
 */
export declare class ErrTransactionOptionsInvalid extends Err {
    constructor(value: number);
}
/**
 * Signals an error related to signing a message (a transaction).
 */
export declare class ErrSignatureCannotCreate extends Err {
    constructor(input: any, inner?: Error);
}
/**
 * Signals an invalid value for the name of a {@link ContractFunction}.
 */
export declare class ErrInvalidFunctionName extends Err {
    constructor();
}
/**
 * Signals a failed operation, since the Timer is already running.
 */
export declare class ErrAsyncTimerAlreadyRunning extends Err {
    constructor();
}
/**
 * Signals a failed operation, since the Timer has been aborted.
 */
export declare class ErrAsyncTimerAborted extends Err {
    constructor();
}
/**
 * Signals a timout for a {@link TransactionWatcher}.
 */
export declare class ErrTransactionWatcherTimeout extends Err {
    constructor();
}
/**
 * Signals an issue related to waiting for a specific transaction status.
 */
export declare class ErrExpectedTransactionStatusNotReached extends Err {
    constructor();
}
/**
 * Signals an issue related to waiting for specific transaction events.
 */
export declare class ErrExpectedTransactionEventsNotFound extends Err {
    constructor();
}
/**
 * Signals a generic error in the context of Smart Contracts.
 */
export declare class ErrContract extends Err {
    constructor(message: string);
}
export declare class ErrContractHasNoAddress extends ErrContract {
    constructor();
}
/**
 * Signals an error thrown by the mock-like test objects.
 */
export declare class ErrMock extends Err {
    constructor(message: string);
}
/**
 * Signals a generic type error.
 */
export declare class ErrTypingSystem extends Err {
    constructor(message: string);
}
/**
 * Signals a missing field on a struct.
 */
export declare class ErrMissingFieldOnStruct extends Err {
    constructor(fieldName: string, structName: string);
}
/**
 * Signals a missing field on an enum.
 */
export declare class ErrMissingFieldOnEnum extends Err {
    constructor(fieldName: string, enumName: string);
}
/**
 * Signals an error when parsing the contract results.
 */
export declare class ErrCannotParseContractResults extends Err {
    constructor(details: string);
}
/**
 * Signals an error when parsing the outcome of a transaction (results and logs).
 */
export declare class ErrCannotParseTransactionOutcome extends Err {
    constructor(transactionHash: string, message: string);
}
/**
 * Signals a generic codec (encode / decode) error.
 */
export declare class ErrCodec extends Err {
    constructor(message: string);
}
/**
 * Signals a generic contract interaction error.
 */
export declare class ErrContractInteraction extends Err {
    constructor(message: string);
}
/**
 * Signals that a method is not yet implemented
 */
export declare class ErrNotImplemented extends Err {
    constructor();
}
/**
 * Signals that the `isCompleted` property is missing on the transaction obect and is needed for the Transaction Watcher
 */
export declare class ErrIsCompletedFieldIsMissingOnTransaction extends Err {
    constructor();
}
/**
 * Signals that the provided token identifier is not valid
 */
export declare class ErrInvalidTokenIdentifier extends Err {
    constructor(message: string);
}
/**
 * Signals a generic bad usage error
 */
export declare class ErrBadUsage extends Err {
    constructor(message: string);
}
/**
 * Signals an invalid inner transaction for relayed transactions
 */
export declare class ErrInvalidInnerTransaction extends Err {
    constructor(message: string);
}
/**
 * Signals an error when parsing the logs of a transaction.
 */
export declare class ErrParseTransactionOutcome extends Err {
    constructor(message: string);
}
/**
 * Signals an error when querying a smart contract.
 */
export declare class ErrSmartContractQuery extends Err {
    returnCode: string;
    constructor(returnCode: string, message: string);
}
/**
 * Signals a wrong mnemonic format.
 */
export declare class ErrWrongMnemonic extends Err {
    constructor();
}
/**
 * Signals a bad mnemonic entropy.
 */
export declare class ErrBadMnemonicEntropy extends Err {
    constructor(inner: Error);
}
/**
 * Signals a bad PEM file.
 */
export declare class ErrBadPEM extends Err {
    constructor(message?: string);
}
/**
 * Signals an error related to signing a message (a transaction).
 */
export declare class ErrSignerCannotSign extends Err {
    constructor(inner: Error);
}
/**
 * Signals a bad address.
 */
export declare class ErrBadAddress extends Err {
    constructor(value: string, inner?: Error);
}
/**
 * Signals an error that happened during a request against the Network.
 */
export declare class ErrNetworkProvider extends Err {
    constructor(url: string, error: string, inner?: Error);
}
/**
 * Signals a generic error in the context of querying Smart Contracts.
 */
export declare class ErrContractQuery extends Err {
    constructor(originalError: Error);
}
/**
 * Signals that the network provider provided is not valid
 */
export declare class ErrInvalidNetworkProviderKind extends Err {
    constructor();
}
/**
 * Signals that the account condition was not reached
 */
export declare class ExpectedAccountConditionNotReachedError extends Err {
    constructor();
}
